@@ -1,5 +1,6 @@
 import dataclasses
-from typing import (
+import re
+from typing import (  # type: ignore
     Any,
     ClassVar,
     Generic,
@@ -8,6 +9,7 @@ from typing import (
     Sequence,
     Type,
     Union,
+    _TypedDictMeta,
 )
 
 from dbdaora import FallbackDataSource, MemoryDataSource
@@ -42,16 +44,21 @@ class MemoryRepository(Generic[Entity, EntityData, FallbackKey]):
         many_key_attrs: Optional[Type[Entity]] = None,
     ):
         super().__init_subclass__()
-        entity_name = getattr(cls, 'entity_name', entity_name)
-        entity_cls = getattr(cls, 'entity_cls', entity_cls)
-        key_attrs = getattr(cls, 'key_attrs', key_attrs)
-        many_key_attrs = getattr(cls, 'many_key_attrs', many_key_attrs)
+        entity_name = getattr(cls, 'entity_name', None) or entity_name
+        entity_cls = getattr(cls, 'entity_cls', None) or entity_cls
+        key_attrs = getattr(cls, 'key_attrs', None) or key_attrs
+        many_key_attrs = getattr(cls, 'many_key_attrs', None) or many_key_attrs
 
         if cls.__name__ not in cls.__skip_cls_validation__:
+            if entity_name is None:
+                entity_name = re.sub(
+                    r'(?<!^)(?=[A-Z])', '_', cls.__name__
+                ).lower()
+                entity_name = entity_name.replace('_repository', '')
+
             missing_attrs = []
 
             for name, value in (
-                ('entity_name', entity_name),
                 ('entity_cls', entity_cls),
                 ('key_attrs', key_attrs),
             ):
@@ -262,7 +269,12 @@ class MemoryRepository(Generic[Entity, EntityData, FallbackKey]):
                 self.entity_name, *query.key_parts
             )
 
-        if isinstance(query, self.get_entity_type(query)):
+        elif isinstance(self.get_entity_type(query), _TypedDictMeta):
+            return self.memory_data_source.make_key(
+                self.entity_name, *self.key_parts(query)
+            )
+
+        elif isinstance(query, self.get_entity_type(query)):
             return self.memory_data_source.make_key(
                 self.entity_name, *self.key_parts(query)
             )
@@ -285,7 +297,12 @@ class MemoryRepository(Generic[Entity, EntityData, FallbackKey]):
                 self.entity_name, *query.key_parts
             )
 
-        if isinstance(query, self.get_entity_type(query)):
+        elif isinstance(self.get_entity_type(query), _TypedDictMeta):
+            return self.fallback_data_source.make_key(
+                self.entity_name, *self.key_parts(query)
+            )
+
+        elif isinstance(query, self.get_entity_type(query)):
             return self.fallback_data_source.make_key(
                 self.entity_name, *self.key_parts(query)
             )
@@ -334,7 +351,12 @@ class MemoryRepository(Generic[Entity, EntityData, FallbackKey]):
                 self.entity_name, 'not-found', *query.key_parts
             )
 
-        if isinstance(query, self.get_entity_type(query)):
+        elif isinstance(self.get_entity_type(query), _TypedDictMeta):
+            return self.memory_data_source.make_key(
+                self.entity_name, 'not-found', *self.key_parts(query)
+            )
+
+        elif isinstance(query, self.get_entity_type(query)):
             return self.memory_data_source.make_key(
                 self.entity_name, 'not-found', *self.key_parts(query)
             )
